# -*- coding: utf-8 -*-


import os
import sys
import unittest

sys.path.append("akamai/edgeauth")
sys.path.append("../akamai/edgeauth")
from edgeauth import EdgeAuth, EdgeAuthError

import requests


if 'TEST_MODE' in os.environ and os.environ['TEST_MODE'] == 'TRAVIS':
    AT_HOSTNAME = os.environ['AT_HOSTNAME']
    AT_ENCRYPTION_KEY = os.environ['AT_ENCRYPTION_KEY']
    AT_TRANSITION_KEY = os.environ['AT_TRANSITION_KEY']
    AT_SALT = os.environ['AT_SALT']
else:
    import secrets
    AT_HOSTNAME = secrets.AT_HOSTNAME
    AT_ENCRYPTION_KEY = secrets.AT_ENCRYPTION_KEY
    AT_TRANSITION_KEY = secrets.AT_TRANSITION_KEY    
    AT_SALT = secrets.AT_SALT

DEFAULT_WINDOW_SECONDS = 500


class TestEdgeAuth(unittest.TestCase):

    def setUp(self):
        # Test for Query String
        self.at = EdgeAuth(**{'key': AT_ENCRYPTION_KEY, 'window_seconds': DEFAULT_WINDOW_SECONDS})
        
        # Test for Cookie
        self.cat = EdgeAuth(key=AT_ENCRYPTION_KEY, algorithm='sha1', window_seconds=DEFAULT_WINDOW_SECONDS)

        # Test for Header
        self.hat = EdgeAuth(key=AT_ENCRYPTION_KEY, algorithm='md5', window_seconds=DEFAULT_WINDOW_SECONDS)

    def _token_setting(self, ttype, escape_early, transition):
        t = None
        if ttype == 'q':
            t = self.at
        elif ttype == 'c':
            t = self.cat
        elif ttype == 'h':
            t = self.hat
        
        if transition:
            t.key = AT_TRANSITION_KEY
        
        t.escape_early = escape_early

    def _queryAssertEqual(self, path, expacted, escape_early=False, transition=False,
                          payload=None, session_id=None, isUrl=True):
        self._token_setting('q', escape_early, transition)
        if isUrl:
            token = self.at.generateToken(url=path, payload=payload, session_id=session_id)
        else:
            token = self.at.generateToken(acl=path, payload=payload, session_id=session_id)
 
        url = "http://{0}{1}{4}{2}={3}".format(AT_HOSTNAME, path, self.at.token_name, token,
            '&' if '?' in path else '?')
        response = requests.get(url)
        self.assertEqual(expacted, response.status_code)
    
    def _cookieAssertEqual(self, path, expacted, escape_early=False, transition=False,
                           payload=None, session_id=None, isUrl=True):
        self._token_setting('c', escape_early, transition)
        if isUrl:
            token = self.cat.generateToken(url=path, payload=payload, session_id=session_id)
        else:
            token = self.cat.generateToken(acl=path, payload=payload, session_id=session_id)

        url = "http://{0}{1}".format(AT_HOSTNAME, path)
        response = requests.get(url, cookies={self.cat.token_name: token})
        self.assertEqual(expacted, response.status_code)

    def _headerAssertEqual(self, path, expacted, escape_early=False, transition=False,
                           payload=None, session_id=None, isUrl=True):
        self._token_setting('h', escape_early, transition)
        if isUrl:
            token = self.hat.generateToken(url=path, payload=payload, session_id=session_id)
        else:
            token = self.hat.generateToken(acl=path, payload=payload, session_id=session_id)

        url = "http://{0}{1}".format(AT_HOSTNAME, path)
        response = requests.get(url, headers={self.hat.token_name: token})
        self.assertEqual(expacted, response.status_code)
        
    def _test_case_set(self, query_path, cookie_path, header_path, escape_early, isUrl):
        # General Test
        self._queryAssertEqual(query_path, 404, escape_early=escape_early, isUrl=isUrl)
        self._cookieAssertEqual(cookie_path, 404, escape_early=escape_early, isUrl=isUrl)
        self._headerAssertEqual(header_path, 404, escape_early=escape_early, isUrl=isUrl)


        if isUrl:
            query_string="?foo=bar&hello=world"
            self._queryAssertEqual(query_path + query_string, 403, escape_early=(False==escape_early), isUrl=isUrl)
            self._cookieAssertEqual(cookie_path + query_string, 403, escape_early=(False==escape_early), isUrl=isUrl)
            self._headerAssertEqual(header_path + query_string, 403, escape_early=(False==escape_early), isUrl=isUrl)

        # Transition Key Test
        self._queryAssertEqual(query_path, 404, transition=True, escape_early=escape_early, isUrl=isUrl)
        self._cookieAssertEqual(cookie_path, 404, transition=True, escape_early=escape_early, isUrl=isUrl)
        self._headerAssertEqual(header_path, 404, transition=True, escape_early=escape_early, isUrl=isUrl)

        # Payload Test
        self._queryAssertEqual(query_path, 404, payload='SOME_PAYLOAD_DATA', escape_early=escape_early, isUrl=isUrl)
        self._cookieAssertEqual(cookie_path, 404, payload='SOME_PAYLOAD_DATA', escape_early=escape_early, isUrl=isUrl)
        self._headerAssertEqual(header_path, 404, payload='SOME_PAYLOAD_DATA', escape_early=escape_early, isUrl=isUrl)

        # Session Id Test
        self._queryAssertEqual(query_path, 404, session_id='SOME_SESSION_ID_DATA', escape_early=escape_early, isUrl=isUrl)
        self._cookieAssertEqual(cookie_path, 404, session_id='SOME_SESSION_ID_DATA', escape_early=escape_early, isUrl=isUrl)
        self._headerAssertEqual(header_path, 404, session_id='SOME_SESSION_ID_DATA', escape_early=escape_early, isUrl=isUrl)
    
    ##########
    # URL TEST
    ##########
    def test_url_escape_on__ignoreQuery_yes(self):
        self._test_case_set("/q_escape_ignore", "/c_escape_ignore", "/h_escape_ignore", escape_early=True, isUrl=True)

    def test_url_escape_off__ignoreQuery_yes(self):
        self._test_case_set("/q_ignore", "/c_ignore", "/h_ignore", escape_early=False, isUrl=True)

    def test_url_escape_on__ignoreQuery_no(self):
        query_path = "/q_escape"
        cookie_path = "/c_escape"
        header_path = "/h_escape"
        self._test_case_set(query_path, cookie_path, header_path, escape_early=True, isUrl=True)

        query_string="?foo=bar&hello=world"
        self._queryAssertEqual(query_path + query_string, 404, escape_early=True, isUrl=True)
        self._cookieAssertEqual(cookie_path + query_string, 404, escape_early=True, isUrl=True)
        self._headerAssertEqual(header_path + query_string, 404, escape_early=True, isUrl=True)

    def test_url_escape_off__ignoreQuery_no(self):
        query_path = "/q"
        cookie_path = "/c"
        header_path = "/h"
        self._test_case_set(query_path, cookie_path, header_path, escape_early=False, isUrl=True)
        
        query_string="?foo=bar&hello=world"
        self._queryAssertEqual(query_path + query_string, 404, escape_early=False, isUrl=True)
        self._cookieAssertEqual(cookie_path + query_string, 404, escape_early=False, isUrl=True)
        self._headerAssertEqual(header_path + query_string, 404, escape_early=False, isUrl=True)
    
    def test_url_query_escape_on__ignore_yes_with_salt(self):
        query_salt_path = "/salt"
        ats = EdgeAuth(key=AT_ENCRYPTION_KEY, salt=AT_SALT, window_seconds=DEFAULT_WINDOW_SECONDS, escape_early=True)
        token = ats.generateToken(url=query_salt_path)
        url = "http://{0}{1}?{2}={3}".format(AT_HOSTNAME, query_salt_path, ats.token_name, token)
        response = requests.get(url)
        self.assertEqual(404, response.status_code)
    ##########
    
    ##########
    # ACL TEST
    ##########
    def test_acl_escape_on__ignoreQuery_yes(self):
        self._test_case_set("/q_escape_ignore", "/c_escape_ignore", "/h_escape_ignore", escape_early=False, isUrl=False)

    def test_acl_escape_off__ignoreQuery_yes(self):
        self._test_case_set("/q_ignore", "/c_ignore", "/h_ignore", escape_early=False, isUrl=False)

    def test_acl_escape_on__ignoreQuery_no(self):
        self._test_case_set("/q_escape", "/c_escape", "/h_escape", escape_early=False, isUrl=False)

    def test_acl_escape_off__ignoreQuery_no(self):
        self._test_case_set("/q", "/c", "/h", escape_early=False, isUrl=False)
    
    def test_acl_asta_escape_on__ignoreQuery_yes(self):
        ata = EdgeAuth(key=AT_ENCRYPTION_KEY, window_seconds=DEFAULT_WINDOW_SECONDS, escape_early=False)
        token = ata.generateToken(acl='/q_escape_ignore/*')
        url = "http://{0}{1}?{2}={3}".format(AT_HOSTNAME, '/q_escape_ignore/hello', ata.token_name, token)
        response = requests.get(url)
        self.assertEqual(404, response.status_code)
    
    def test_acl_deli_escape_on__ignoreQuery_yes(self):
        atd = EdgeAuth(key=AT_ENCRYPTION_KEY, window_seconds=DEFAULT_WINDOW_SECONDS, escape_early=False)
        acl = ['/q_escape_ignore', '/q_escape_ignore/*']
        token = atd.generateToken(acl=EdgeAuth.ACL_DELIMITER.join(acl))
        url = "http://{0}{1}?{2}={3}".format(AT_HOSTNAME, '/q_escape_ignore', atd.token_name, token)
        response = requests.get(url)
        self.assertEqual(404, response.status_code)

        url = "http://{0}{1}?{2}={3}".format(AT_HOSTNAME, '/q_escape_ignore/world/', atd.token_name, token)
        response = requests.get(url)
        self.assertEqual(404, response.status_code)
    ##########

    def test_times(self):
        if not (sys.version_info[0] == 2 and sys.version_info[1] <= 6):
            att = EdgeAuth(key=AT_ENCRYPTION_KEY, window_seconds=DEFAULT_WINDOW_SECONDS, escape_early=False)
            # start_time
            with self.assertRaises(EdgeAuthError):
                att.generateToken(start_time=-1)
            
            with self.assertRaises(EdgeAuthError):
                att.generateToken(start_time="hello")
            
            # end_time
            with self.assertRaises(EdgeAuthError):
                att.generateToken(end_time=-1)

            with self.assertRaises(EdgeAuthError):
                att.generateToken(end_time="hello")
            
            # window_seconds
            with self.assertRaises(EdgeAuthError):
                att.generateToken(window_seconds=-1)

            with self.assertRaises(EdgeAuthError):
                att.generateToken(window_seconds="hello")
    

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-


import binascii
import hashlib
import hmac
import os
import re
import sys
import time
if sys.version_info[0] >= 3:
    from urllib.parse import quote_plus
else:
    from urllib import quote_plus

# Force the local timezone to be GMT.
os.environ['TZ'] = 'GMT'


class EdgeAuthError(Exception):
    def __init__(self, text):
        self._text = text

    def __str__(self):
        return 'EdgeAuthError:{0}'.format(self._text)

    def _getText(self):
        return str(self)
    text = property(_getText, None, None,
        'Formatted error text.')


class EdgeAuth:
    ACL_DELIMITER='!'
    
    def __init__(self, token_type=None, token_name='__token__',
                 key=None, algorithm='sha256', salt=None,
                 start_time=None, end_time=None, window_seconds=None,
                 field_delimiter='~', escape_early=False, verbose=False):
        
        self.token_type = token_type
        self.token_name = token_name
        self.start_time = start_time
        self.end_time = end_time
        self.window_seconds = window_seconds
        if key is None or len(key) <= 0:
            raise EdgeAuthError('You must provide a secret in order to '
                'generate a new token.')
        self.key = key
        self.algorithm = algorithm
        self.salt = salt
        self.field_delimiter = field_delimiter
        self.escape_early = escape_early
        self.verbose = verbose

    def _escapeEarly(self, text):
        if self.escape_early:
            def toLower(match):
                return match.group(1).lower()
            return re.sub(r'(%..)', toLower, quote_plus(text))
        else:
            return text

    def generateToken(self, url=None, acl=None, start_time=None, 
                       end_time=None, window_seconds=None,
                       ip=None, payload=None, session_id=None):
        if not start_time:
            start_time = self.start_time
        if not end_time:
            end_time = self.end_time
        if not window_seconds:
            window_seconds = self.window_seconds

        if str(start_time).lower() == 'now':
            start_time = int(time.mktime(time.gmtime()))
        elif start_time:
            try:
                if int(start_time) <= 0:
                    raise EdgeAuthError('start_time must be ( > 0 )')    
            except:
                raise EdgeAuthError('start_time must be numeric or now')

        if end_time:
            try:
                if int(end_time) <= 0:
                    raise EdgeAuthError('end_time must be ( > 0 )')
            except:
                raise EdgeAuthError('end_time must be numeric')

        if window_seconds:
            try:
                if int(window_seconds) <= 0:
                    raise EdgeAuthError('window_seconds must be ( > 0 )')    
            except:
                raise EdgeAuthError('window_seconds must be numeric')
                
        if end_time is None:
            if window_seconds:
                if start_time is None:
                    # If we have a window_seconds without a start time,
                    # calculate the end time starting from the current time.
                    end_time = int(time.mktime(time.gmtime())) + \
                        window_seconds
                else:
                    end_time = start_time + window_seconds
            else:
                raise EdgeAuthError('You must provide an expiration time or '
                    'a duration window ( > 0 )')
        
        if start_time and (end_time <= start_time):
            raise EdgeAuthError('Token will have already expired.')
        
        if (not acl and not url) or (acl and url):
            raise EdgeAuthError('You must provide a URL or an ACL')

        if self.verbose:
            print('''
Akamai Token Generation Parameters
Token Type      : {0}
Token Name      : {1}
Start Time      : {8}
End Time        : {9}
Window(seconds) : {10}
IP              : {11}
URL             : {12}
ACL             : {13}
Key/Secret      : {2}
Payload         : {14}
Algo            : {3}
Salt            : {4}
Session ID      : {15}
Field Delimiter : {5}
ACL Delimiter   : {6}
Escape Early    : {7}
Generating token...'''.format(self.token_type if self.token_type else '', #0
                            self.token_name if self.token_name else '', #1
                            self.key if self.key else '', #2
                            self.algorithm if self.algorithm else '', #3
                            self.salt if self.salt else '', #4
                            self.field_delimiter if self.field_delimiter else '', #5
                            ACL_DELIMITER if ACL_DELIMITER else '', #6
                            self.escape_early if self.escape_early else '', #7
                            start_time if start_time else '', #8
                            end_time if end_time else '', #9
                            window_seconds if window_seconds else '', #10
                            ip if ip else '', #11
                            url if url else '', #12
                            acl if acl else '', #13
                            payload if payload else '', #14
                            session_id if session_id else '')) #15

        hash_source = []
        new_token = []

        if ip:
            new_token.append('ip={0}'.format(self._escapeEarly(ip)))

        if start_time:
            new_token.append('st={0}'.format(start_time))

        new_token.append('exp={0}'.format(end_time))

        if acl:
            new_token.append('acl={0}'.format(acl))

        if session_id:
            new_token.append('id={0}'.format(self._escapeEarly(session_id)))

        if payload:
            new_token.append('data={0}'.format(self._escapeEarly(payload)))

        hash_source = list(new_token)
        if url and not acl:
            hash_source.append('url={0}'.format(self._escapeEarly(url)))

        if self.salt:
            hash_source.append('salt={0}'.format(self.salt))

        if self.algorithm.lower() not in ('sha256', 'sha1', 'md5'):
            raise EdgeAuthError('Unknown algorithm')

        token_hmac = hmac.new(
            binascii.a2b_hex(self.key.encode()),
            self.field_delimiter.join(hash_source).encode(),
            getattr(hashlib, self.algorithm.lower())).hexdigest()
        new_token.append('hmac={0}'.format(token_hmac))

        return self.field_delimiter.join(new_token)


# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


with open('README.rst', 'r') as f:
    readme = f.read()

setup (
    name = 'akamai-edgeauth',
    version = '0.1.0',
    description = 'Akamai Edge Authorization Token for Property Manager Behavior',
    long_description = readme,
    namespace_packages=['akamai'],
    packages=find_packages(exclude=['spike']),
    author = 'Astin Choi',
    author_email = 'asciineo@gmail.com',
    url = 'https://github.com/AstinCHOI/Akamai-EdgeAuth-Python',
    license='Apache 2.0',
    keywords='edgeauth auth token akamai openapi open api',
    classifiers=(
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ),
)